<?php
// $Id: event.inc,v 1.5 2006/01/09 13:16:48 thox Exp $

/**
 * Implementation of hook_access().
 */
function scouting_event_access($op, $node) {
  return _scouting_access($op, $node);
}

/**
 * Implementation of hook_form().
 */
function scouting_event_form(&$node, &$param) {
  scouting_custom_theme();

  $members = array();
  $rs = db_query('SELECT n.nid, p.surname, p.forename, p.prefname, st.name AS "section", mt.name AS "member_type" FROM {node} n, {scouting_person} p, {scouting_section} s, {scouting_section_type} st, {scouting_membership} m, {scouting_membership_type} mt WHERE n.nid = m.nid AND m.section_id = s.nid AND s.section_type_id = st.id AND p.nid = m.person_id AND (m.date_left = 0 OR m.date_left > %d) AND mt.id = m.member_type_id AND m.member_type_id <> 3 AND m.nid NOT IN (SELECT member_id FROM {scouting_event_participant} sp WHERE sp.event_id = %d) ORDER BY "member_type", "section", p.surname, p.forename', time(), $node->nid);
  while ($r = db_fetch_object($rs)) {
    $members[$r->member_type.' - '.$r->section][$r->nid] .= scouting_person_display_name($r);
  }

  $form['title'] = array('#type' => 'textfield', '#title' => t('Event Name'), '#required' => TRUE, '#default_value' => $node->title);
  $form['event_type_id'] = array('#type' => 'select', '#title' => t('Type'), '#default_value' => $node->event_type_id, '#options' => scouting_event_types(), '#required' => TRUE, '#weight' => 0);
  //$form['event_date'] = array('#type' => 'textfield', '#title' => t('Date'), '#size' => 20, '#maxlength' => 32, '#default_value' => scouting_format_date($node->event_date), '#weight' => 0);
  $form['nights_away'] = array('#type' => 'textfield', '#title' => t('Nights Away'), '#size' => 3, '#maxlength' => 3, '#default_value' => $node->nights_away, '#description' => t('Only applicable for camps/sleepovers/nights away.'), '#weight' => 0);
  $form['body'] = array('#type' => 'textarea', '#title' => t('Notes'), '#default_value' => $node->body, '#weight' => 1, '#rows' => 5);
  $form['format'] = filter_form($node->format);
  $form['format']['#weight'] = 1.1;

  $form['participants'] = array('#type' => 'fieldset', '#title' => t('Participants'), '#weight' => 2, '#collapsible' => true);
  $form['participants']['existing_participants'] = array('#value' => scouting_event_participants_table($node->nid, true));
  $form['participants']['new_participants'] = array('#type' => 'select', '#title' => t('Add Participants'), '#default_value' => NULL, '#options' => $members, '#multiple' => TRUE, '#extra' => 'size="10"', '#description' => t('Choose one or more members to add to this event'));

  return $form;
}

/**
 * Implementation of hook_load().
 */
function scouting_event_load($node) {
  return db_fetch_object(db_query('SELECT e.*, et.name AS "event_type" FROM {scouting_event} e, {scouting_event_type} et WHERE e.event_type_id = et.id AND e.nid = %d', $node->nid));
}

/**
 * Implementation of hook_insert().
 */
function scouting_event_insert($node) {
  db_query('INSERT INTO {scouting_event} (nid, nights_away, event_date, event_type_id) VALUES (%d, %d, %d, %d)', $node->nid, $node->nights_away, $node->event_start, $node->event_type_id);
  scouting_event_save_participants($node);
}

/**
 * Implementation of hook_delete().
 */
function scouting_event_delete($node) {
  db_query('DELETE FROM {scouting_event} WHERE nid = %d', $node->nid);
  db_query('DELETE FROM {scouting_event_participant} WHERE event_id = %d', $node->nid);
}

/**
 * Implementation of hook_update().
 */
function scouting_event_update($node) {
  db_query('UPDATE {scouting_event} SET nights_away = %d, event_date = %d, event_type_id = %d WHERE nid = %d', $node->nights_away, $node->event_start, $node->event_type_id, $node->nid);
  scouting_event_save_participants($node);
}

function scouting_event_save_participants($node) {
  if ($node->new_participants) {
    foreach ($node->new_participants as $participant) {
      db_query('INSERT INTO {scouting_event_participant} (event_id,member_id) VALUES (%d, %d)', $node->nid, $participant);
    }
  }
}

/**
 * Implementation of hook_view().
 */
function scouting_event_view(&$node, $teaser = false, $page = false) {
  scouting_custom_theme();

  if (!user_access('access scouting content')) {
    return;
  }

  $info = array(
    t('Date') => scouting_format_date($node->event_date),
    t('Type') => $node->event_type,
    t('Nights Away') => $node->nights_away,
  );

  $output = theme('fieldset', array('#title' => t('Details'), '#children' => theme('scouting_definition_list', $info)));

  $output .= check_markup($node->body, $node->format);

  if (!$teaser) {
    $output .= scouting_event_participants_table($node->nid);
  }

  $node->body = $output;
}

/**
 * Shows a list of recent events
 */
function scouting_event_list_page() {
  $header = array(
      array('data' => t('Event name'), 'field' => 'n.title'),
      array('data' => t('Type'), 'field' => 'type'),
      array('data' => t('Date'), 'field' => 'e.event_date', 'sort' => 'desc'),
      array('data' => t('Nights Away'), 'field' => 'e.nights_away'),
      array('data' => t('Participants'), 'field' => 'participants'),
  );

  $sql = 'SELECT e.nid, n.title, et.name AS "type", e.event_date AS "date", e.nights_away, COUNT(*) AS "participants" FROM {scouting_event} e, {node} n, {scouting_event_type} et, {scouting_event_participant} ep WHERE et.id = e.event_type_id AND e.nid = n.nid AND ep.event_id = e.nid GROUP BY e.nid'.tablesort_sql($header);

  $rs = pager_query($sql, 20);

  $rows = array();
  while ($event = db_fetch_object($rs)) {
    $rows[] = array(
      l($event->title, 'node/'.$event->nid),
      $event->type,
      scouting_format_date($event->date),
      $event->nights_away,
      $event->participants,
    );
  }

  $output = theme('table', $header, $rows);

  if ($pager = theme('pager', NULL, 25, 0, tablesort_pager())) {
    $output .= $pager;
  }

  return $output;
}

/**
 * Removes a participating Scout member from an event
 */
function scouting_event_remove_participant($member_id, $event_id, $js = FALSE) {
  db_query('DELETE FROM {scouting_event_participant} WHERE event_id = %d AND member_id = %d', $event_id, $member_id);

  if ($js) {
    exit();
  }
  else {
    drupal_goto('node/'.$event_id.'/edit');
  }
}

function scouting_event_print_page($event_nid) {
  $event = node_load(array('nid' => $event_nid));

  $header = array(
    '#',
    'Name',
    'DOB',
    'Telephone',
    'Mobile',
    //'Member Type',
    //'Section',
    'Address 1',
    'Address 2',
    'Town',
    'Medical notes');

  $rows = array();

  $result = db_query('SELECT n.body, p.*, mt.name as "member_type", s.title AS "section" FROM {scouting_event_participant} ep, {scouting_membership} m, {scouting_membership_type} mt, {scouting_person} p, {node} s, {node_revisions} n WHERE n.nid = p.nid AND s.nid = m.section_id AND ep.member_id = m.nid AND p.nid = m.person_id AND mt.id = m.member_type_id AND ep.event_id=%d ORDER BY m.member_type_id, p.surname, p.forename', $event_nid);

  $i = 1;
  while ($participant = db_fetch_object($result)) {
    $rows[] = array(
      $i++,
      scouting_person_display_name($participant),
      scouting_format_date($participant->dob),
      $participant->phone_number,
      $participant->mobile_number,
      //$participant->member_type,
      //$participant->section,
      $participant->address1,
      $participant->address2,
      $participant->town,
      $participant->body,
    );
  }

  $output = '<style type="text/css">
dt, dd { display: inline; margin: 0; padding: 0 .2em; }
dt { font-weight: bold; }
dd { padding-right: 2em; }
h1 { margin: 0; }
h1, dl { margin: 0 auto; text-align: center; }
table { border-collapse: collapse; }
td, th { border: 1px solid #ddd; padding: .1em; font-size: 80%; font-family: sans-serif; }
th { background: #eee; font-weight: bold; }
</style>';

  $output .= '<h1>'.$event->title.'</h1>';

  $info = array(
    'Type' => $event->event_type,
    //'Date' => scouting_format_date($event->event_date),
    'Nights Away' => $event->nights_away
  );

  $output .= theme('scouting_definition_list', $info);

  $output .= theme('table', $header, $rows);

  echo $output;
}

/**
 * Creates a table of participants for an event
 *
 * @param $event_nid
 *   An integer representing the nid of the event
 * @param $editable
 *   Whether the table is edittable with AJAX or not
 * @return
 *   A form group containing the table of participants
 */
function scouting_event_participants_table($event_nid, $editable = FALSE) {
  $header = array(
    array('data' => t('Name'), 'field' => 'p.surname'),
    array('data' => t('Member Type'), 'field' => 'mt.name'),
    array('data' => t('Section'), 'field' => 's.title')
  );
  if ($editable) {
    $header[] = 'Operation';
  }
  if ($event_nid > 0) {
    drupal_add_js('misc/drupal.js');
    $rs = db_query('SELECT p.nid, p.surname, p.forename, mt.name as "member_type", m.member_type_id, m.nid AS "member_id", s.title AS "section", s.nid AS "section_id", p.prefname FROM {scouting_event_participant} ep, {scouting_membership} m, {scouting_membership_type} mt, {scouting_person} p, {node} s WHERE s.nid = m.section_id AND ep.member_id = m.nid AND p.nid = m.person_id AND mt.id = m.member_type_id AND ep.event_id=%d'.tablesort_sql($header), $event_nid);
    while ($person = db_fetch_object($rs)) {
      $row = array(
        l(scouting_person_display_name($person), 'node/'.$person->nid),
        l($person->member_type, 'node/'.$person->member_id),
        l($person->section, 'node/'.$person->section_id)
      );
      if ($editable) {
        $row[] = l(t('Remove'), 'scouting/remove_participant/' . $person->member_id.'/'.$event_nid, array('onclick' => 'HTTPGet(this.href + "/js"); removeNode(this.parentNode.parentNode); return false;'));
      }
      $rows[] = $row;
    }
  }
  if (count($rows) == 0) {
    $rows[] = array(array(
      'data' => t('None'),
      'colspan' => count($header)
    ));
  }

  $output = theme('table', $header, $rows);

  return $output;
}

/**
 * Returns a nested (2 level) array of event types (e.g. cub badges, scout badges, staged badges)
 */
function scouting_event_types() {
  static $types;

  if (!$types) {
    $rs = db_query('SELECT e_type.id, e_type.name, e_cat.name AS "category" FROM {scouting_event_type} e_type, {scouting_event_category} e_cat WHERE e_type.eventcategoryid = e_cat.id');
    while ($r = db_fetch_object($rs)) {
      $types[$r->category][$r->id] = $r->name;
    }
  }

  return  $types;
}

?>