<?php
// $Id: member.inc,v 1.5 2005/12/17 23:12:31 thox Exp $

/**
 * Implementation of hook_access().
 */
function scouting_membership_access($op, $node) {
  return _scouting_access($op, $node);
}

/**
 * Implementation of hook_form().
 */
function scouting_membership_form(&$node, &$param) {
  scouting_custom_theme();

  if ($_GET['personid'] > 0) {
    $node->person_id = (int) $_GET['personid'];
    $node->member_type_id = 2;
    $node->date_joined = time();
  }

  $form['member_name'] = array( '#type' => 'textfield', '#title' => t('Member Name'), '#size' => 65, '#maxlength' => 128, '#required' => TRUE, '#default_value' => scouting_person_name_from_id($node->person_id), '#autocomplete_path' => 'scouting/people/autocomplete', '#weight' => -1);
  $form['member_type_id'] = array( '#type' => 'select', '#title' => t('Member Type'), '#default_value' => $node->member_type_id, '#options' => scouting_membership_types(), '#weight' => -1);
  $form['section_id'] = array( '#type' => 'select', '#title' => t('Section'), '#default_value' => $node->section_id, '#options' => scouting_sections(), '#weight' => -1);

  // @@ TODO: fit taxonomy into here rather than default location

  $form['dates'] = array( '#type' => 'fieldset', '#title' => t('Dates'), '#weight' => -1);
  $form['dates']['date_joined'] = array( '#type' => 'textfield', '#title' => t('Date joined'), '#size' => 20, '#maxlength' => 32, '#default_value' => scouting_format_date($node->date_joined));
  $form['dates']['date_invested'] = array( '#type' => 'textfield', '#title' => t('Date invested'), '#size' => 20, '#maxlength' => 32, '#default_value' => scouting_format_date($node->date_invested));
  $form['dates']['date_left'] = array( '#type' => 'textfield', '#title' => t('Date left'), '#size' => 20, '#maxlength' => 32, '#default_value' => scouting_format_date($node->date_left));

  $form['body'] = array( '#type' => 'textarea', '#title' => t('Reason for leaving'), '#default_value' => $node->body, '#weight' => 0, '#rows' => 5,);

  return $form;
}

/**
 * Implementation of hook_load().
 */
function scouting_membership_load($node) {
  return db_fetch_object(db_query('SELECT m.*, p.forename, p.surname, p.prefname, s.title AS "section", mt.name AS "member_type" FROM {scouting_membership} m, {scouting_membership_type} mt, {scouting_person} p, {node} s WHERE m.nid = %d AND p.nid = m.person_id AND s.nid = m.section_id AND m.member_type_id = mt.id', $node->nid));
}

/**
 * Implementation of hook_insert().
 */
function scouting_membership_insert($node) {
  db_query('INSERT INTO {scouting_membership} (nid, person_id, member_type_id, section_id, date_joined, date_invested, date_left) VALUES (%d, %d, %d, %d, %d, %d, %d)', $node->nid, $node->person_id, $node->member_type_id, $node->section_id, $node->date_joined, $node->date_invested, $node->date_left);
}

/**
 * Implementation of hook_update().
 */
function scouting_membership_update($node) {
  db_query('UPDATE {scouting_membership} SET person_id=%d,member_type_id=%d,section_id=%d,date_joined=%d,date_invested=%d,date_left=%d WHERE nid = %d', $node->person_id, $node->member_type_id, $node->section_id, $node->date_joined, $node->date_invested, $node->date_left, $node->nid);
}

/**
 * Implementation of hook_delete().
 */
function scouting_membership_delete($node) {
  db_query('DELETE FROM {scouting_membership} WHERE nid = %d', $node->nid);
}

/**
 * Implementation of hook_submit().
 */
function scouting_membership_submit(&$node) {
  $node->date_joined = scouting_strtotime($node->date_joined);
  $node->date_invested = scouting_strtotime($node->date_invested);
  $node->date_left = scouting_strtotime($node->date_left);
  $node->title = $node->member_name;
}

/**
 * Implementation of hook_validate().
 */
function scouting_membership_validate(&$node) {
  if ($node->member_name && !$node->person_id) {
    $node->person_id = scouting_person_id_from_name($node->member_name);
    if (!$node->person_id) {
      form_set_error('member_name', t('Could not find a Scouting person with the given name.'));
    }
  }
}

/**
 * Implementation of hook_view().
 */
function scouting_membership_view(&$node, $teaser = false, $page = false) {
  scouting_custom_theme();

  // Member details
  $info = array(
    t('Person Record') => l(scouting_person_display_name($node), 'node/'.$node->person_id),
    t('Member Type') => $node->member_type,
    t('Section') => l($node->section, 'node/'.$node->section_id),
    t('Joined') => scouting_format_date($node->date_joined),
    t('Invested') => scouting_format_date($node->date_invested),
    t('Left') => scouting_format_date($node->date_left),
    t('Duration') => $node->date_left && $node->date_joined ? format_interval($node->date_left - $node->date_joined) : '',
    t('Reason for leaving') => check_markup($node->body, $node->format),
    t('Last updated') => scouting_format_date($node->changed),
  );
  $output = theme('fieldset', array('#title' => t('Details'), '#children' => theme('scouting_definition_list', $info)));

  // Sortable event / badges table
  $header = array(
    array('data' => t('Event name'), 'field' => 'n.title'),
    array('data' => t('Type'), 'field' => 'e.event_type'),
    array('data' => t('Date'), 'field' => 'e.event_date', 'sort' => 'desc'),
    array('data' => t('Nights Away'), 'field' => 'e.nights_away'),
  );

  $sql = '
SELECT
  n.nid,
  n.title,
  et.name AS "type",
  e.nights_away,
  e.event_date
FROM
  {scouting_event_participant} ep,
  {node} n,
  {scouting_event} e,
  {scouting_event_type} et
WHERE
  ep.member_id = %d
  AND n.nid = e.nid
  AND e.nid = ep.event_id
  AND et.id = e.event_type_id
'.tablesort_sql($header);

  $rs = db_query($sql, $node->nid);

  $rows = array();
  $total_nights_away = 0;
  while ($event = db_fetch_object($rs)) {
    $rows[] = array(
      l($event->title, 'node/'.$event->nid),
      $event->type,
      scouting_format_date($event->event_date),
      $event->nights_away,
    );
    if ($event->event_date < time()) {
      $total_nights_away += $event->nights_away;
    }
  }

  $group = theme('table', $header, $rows);
  $group .= '<p>Total nights away: <strong>'.$total_nights_away.'</strong>.</p>';

  $output .= theme('fieldset', array('#title' => t('Badges / Events'), '#children' => $group));

  $node->body = $output;
}

function scouting_members_overage() {

  // @@ TODO: Cron check with this query, update member to "Inactive Member" and e-mail leader.

  $sql = 'SELECT
    p.nid, p.forename, p.surname, p.prefname, p.dob,
    (31557600 * st.absmaxage) - ('.time().' - p.dob) as "abs_time_left"
  FROM
    {scouting_person} p,
    {scouting_membership} m,
    {scouting_section} ss,
    {scouting_section_type} st
  WHERE
    p.nid = m.person_id
    AND m.section_id = ss.nid
    AND ss.section_type_id = st.id
    AND m.member_type_id = 2
    AND p.dob > 0
  AND (31557600 * st.absmaxage) - ('.time().' - p.dob) < 0';

  /*
    [Leader Name],

    [Scout Name] is now [age] and is too old to be a member of [Section Name]. [He/She] has been changed to an inactive member with a leaving date of [Today].

    Please update their membership record with details on why they left and, if possible, correct the leaving date to be more accurate:

    [link to member]

    This message was automatically sent by the Scouting module of the [website name] website. Please do not reply to this e-mail.

  */
}

/**
 * Displays a filtered, paged, sorted table of members with as much information as possible (including taxonomy)
 */
function scouting_membership_list_page() {
  scouting_custom_theme();

  // Construct filter form
  $filters = &$_SESSION['scouting_members_filter'];
  $edit = $_POST['edit'];
  $op = $_POST['op'];

  if (!isset($_SESSION['scouting_members_filter']) || !is_array($filters) || $op == t('Reset')) {
    // some filter defaults
    $filters = array(
      'columns' => array('membertype', 'email', 'nightsaway'),
      'limit' => 50,
      'onlyactive' => true,
    );
  }

  if ($op == t('Filter')) {
    $filters['member_type'] = intval($edit['member_type']);
    $filters['section'] = intval($edit['section']);
    $filters['limit'] = intval($edit['limit']);
    $filters['onlyactive'] = $edit['onlyactive'];
    $filters['columns'] = array_keys($edit['columns']);
  }
  elseif ($_GET['section']) {
    $filters['section'] = intval($_GET['section']);
    drupal_goto('scouting/members');
  }

  $form['c'] = array('#type' => 'fieldset', '#title' => t('Filters'), '#collapsible' => true);
  $form['c']['m1'] = array('#value' => '<div style="float: right; margin-right: 1em; ">');
  $form['c']['member_type'] = array('#type' => 'select', '#title' => t('Member type'), '#default_value' => $filters['member_type'], '#options' => scouting_membership_types(TRUE));
  $form['c']['section'] = array('#type' => 'select', '#title' => t('Section'), '#default_value' => $filters['section'], '#options' => scouting_sections(TRUE));
  $form['c']['limit'] = array('#type' => 'select', '#title' => t('Number to show'), '#default_value' => $filters['limit'], '#options' => drupal_map_assoc(array(10, 25, 50, 100)));
  $form['c']['onlyactive'] = array('#type' => 'checkbox', '#title' => t('Hide members that have left'), '#default_value' => $filters['onlyactive']);
  $form['c']['m2'] = array('#value' => '</div>');
  $options = array(
    'section' => 'Section',
    'membertype' => 'Member type',
    'dates' => 'Member dates',
    'phone' => 'Phone numbers',
    'email' => 'Email address',
    'address' => 'Address',
    'age' => 'Age & DOB',
    'timeleft' => 'Time left in section',
    'nightsaway' => 'Nights Away',
  );
  $form['c']['columns'] = array('#type' => 'checkboxes', '#title' => t('Columns to show'), '#options' => $options, '#default_value' => $filters['columns']);
  $form['c']['submit'] = array('#type' => 'button', '#value' => t('Filter'));
  $form['c']['reset'] = array('#type' => 'button', '#value' => t('Reset'));
  $output = drupal_get_form('scouting_membership_filters', $form);

  $showall = ($_GET['show'] == 'all');

  // Construct table headers and SQL query
  $header = array(
    array('data' => t('Surname'), 'field' => 'p.surname'),
    array('data' => t('Forename'), 'field' => 'p.forename')
  );
  if ($showall || in_array('section', $filters['columns'])) {
    $header[] = array('data' => t('Section'), 'field' => 'ss.title');
  }
  if ($showall || in_array('membertype', $filters['columns'])) {
    $header[] = array('data' => t('Member Type'), 'field' => 'mt.name');
  }
  if ($showall || in_array('dates', $filters['columns'])) {
    $header[] = array('data' => t('Date Joined'), 'field' => 'm.date_joined');
    $header[] = array('data' => t('Date Invested'), 'field' => 'm.date_invested');
    $header[] = array('data' => t('Date Left'), 'field' => 'm.date_left');
  }

  $query->joins = '{node} n, {scouting_person} p, {scouting_membership} m, {scouting_membership_type} mt, {node} ss';
  $query->selects = 'DISTINCT(m.nid), n.nid AS "member_id", m.person_id, p.*, m.member_type_id, m.date_joined, m.date_invested, m.date_left, mt.name AS "member_type_name", mt.level AS "member_level", ss.title AS "section", m.section_id';
  $query->wheres[] = 'm.nid = n.nid';
  $query->wheres[] = 'm.person_id = p.nid';
  $query->wheres[] = 'm.member_type_id=mt.id';
  $query->wheres[] = 'n.type="scouting_membership"';
  $query->wheres[] = 'ss.type="scouting_section"';
  $query->wheres[] = 'ss.nid=m.section_id';

  if ($showall || in_array('phone', $filters['columns'])) {
    $header[] = array('data' => t('Phone'), 'field' => 'p.phone_number');
    $header[] = array('data' => t('Mobile'), 'field' => 'p.mobile_number');
  }
  if ($showall || in_array('email', $filters['columns'])) {
    $header[] = array('data' => t('Email'), 'field' => 'p.email');
  }

  if ($showall || in_array('address', $filters['columns'])) {
    $header[] = array('data' => t('Address 1'), 'field' => 'p.address1');
    $header[] = array('data' => t('Address 2'), 'field' => 'p.address2');
    $header[] = array('data' => t('Town'), 'field' => 'p.town');
    $header[] = array('data' => t('County'), 'field' => 'p.county');
    $header[] = array('data' => t('Postcode'), 'field' => 'p.postcode');
  }

  if ($showall || in_array('age', $filters['columns'])) {
    $header[] = array('data' => t('Age'), 'field' => 'age');
    $header[] = array('data' => t('DOB'), 'field' => 'p.dob');
    $query->selects .= ', p.dob, '.time().' - p.dob AS "age"';
  }

  if ($showall || in_array('timeleft', $filters['columns'])) {
    $header[] = array('data' => t('Time Left'), 'field' => 'time_left');
    $header[] = array('data' => t('Abs. Time Left'), 'field' => 'abs_time_left');
    $query->joins .= ',{scouting_section_type} st, {scouting_section} sec';
    $query->selects .= ', (31557600 * st.maxage) - ('.time().' - p.dob) AS "time_left", (31557600 * st.absmaxage) - ('.time().' - p.dob) AS "abs_time_left"';
    $query->wheres[] = 'st.id = sec.section_type_id';
    $query->wheres[] = 'ss.nid = sec.nid';
  }

  if ($showall || $filters['onlyactive']) {
    $query->wheres[] = 'm.date_left=0';
  }

  if ($filters['member_type'] > 0) {
    $query->wheres[] = 'mt.id='.$filters['member_type'];
  }

  if ($showall || in_array('nightsaway', $filters['columns'])) {
    $header[] = array('data' => t('Nights Away'), 'field' => 'nights_away');
    /*
    $query->joins .= ',{scouting_event_participant} sp, {scouting_event} e';
    $query->wheres[] = 'sp.event_id = e.nid';
    $query->wheres[] = 'sp.member_id = m.nid';
    $query->selects .= ', SUM(e.nights_away) AS "nights_away"';
    $query->groupby[] = 'p.nid';
    */
    $query->selects .= ', (SELECT SUM(e.nights_away) FROM {scouting_event_participant} sp, {scouting_event} e, {scouting_membership} sm WHERE sp.event_id = e.nid AND sp.member_id = sm.nid AND sm.person_id = p.nid AND e.event_date < '.time().') AS "nights_away"';
  }

  if ($filters['section'] > 0) {
    $query->wheres[] = 'm.section_id='.$filters['section'];
  }

  $vocabs = module_invoke('taxonomy', 'get_vocabularies', 'scouting_membership');
  if ($vocabs) {
    foreach ($vocabs as $index => $vocab) {
      if ($vocab->required && !$vocab->multiple) {
        $vids[$index] = $vocab->vid;
        $header[] = array('data'=>$vocab->name,'field'=> 'td'.$index);
      }
    }
    foreach ($vids as $index => $vid) {
      $query->selects .= ',td'. $index .'.name AS td'. $index;
      $query->joins .= ' INNER JOIN {term_node} tn'. $index .' ON n.nid = tn'. $index .'.nid';
      $query->joins .= ' INNER JOIN {term_data} td'. $index .' ON tn'. $index .'.tid = td'. $index .'.tid';
      $query->wheres[] = 'td'. $index .'.vid = '.$vid;
    }
  }


  $sql = 'SELECT '.$query->selects.' FROM '.$query->joins.' WHERE '.implode(' AND ', $query->wheres).($query->groupby ? ' GROUP BY ' . implode($query->groupby, ', ') : '').tablesort_sql($header);

  if ($_GET['mode'] == 'csv' || $filters['limit'] == 0) {
    $result = db_query($sql);
  }
  else {
    $result = pager_query($sql, $filters['limit'], 'members', 'SELECT COUNT(*) FROM (SELECT DISTINCT(m.nid) FROM '.$query->joins.' WHERE '.implode(' AND ', $query->wheres).($query->groupby ? ' GROUP BY ' . implode($query->groupby, ', ') : '').') AS temp');
  }


  // Construct table of results

  global $pager_total_items;
  $output .= '<p>Found '.$pager_total_items['members'].' matching members.</p>';

  while ($node = db_fetch_object($result)) {
    $row = array(
      l($node->surname, 'node/'.$node->person_id),
      $node->forename,
    );
    if ($showall || in_array('section', $filters['columns'])) {
      $row[] = l($node->section, 'node/'.$node->section_id);
    }
    if ($showall || in_array('membertype', $filters['columns'])) {
      $row[] = l($node->member_type_name, 'node/'.$node->member_id);
    }
    if ($showall || in_array('dates', $filters['columns'])) {
      $row[] = scouting_format_date($node->date_joined);
      $row[] = scouting_format_date($node->date_invested);
      $row[] = scouting_format_date($node->date_left);
    }
    if ($showall || in_array('phone', $filters['columns'])) {
      $row[] = $node->phone_number;
      $row[] = $node->mobile_number;
    }
    if ($showall || in_array('email', $filters['columns'])) {
      $row[] = $node->email;
    }
    if ($showall || in_array('address', $filters['columns'])) {
      $row[] = $node->address1;
      $row[] = $node->address2;
      $row[] = $node->town;
      $row[] = $node->county;
      $row[] = $node->postcode;
    }
    if ($showall || in_array('age', $filters['columns'])) {
      $row[] = $node->dob ? format_interval($node->age, 1) : '';
      $row[] = $node->dob ? scouting_format_date($node->dob) : '';
    }
    if ($showall || in_array('timeleft', $filters['columns'])) {
      $row[] = $node->member_type_id == 2 ? ($node->dob ? format_interval($node->time_left) : '') : 'N/A';
      $row[] = $node->member_type_id == 2 ? ($node->dob ? format_interval($node->abs_time_left) : '') : 'N/A';
    }
    if ($showall || in_array('nightsaway', $filters['columns'])) {
      $row[] = $node->nights_away;
    }
    if ($vids) {
      foreach ($vids as $index=>$vid) {
        $term_name = 'td'.$index;
        $row[] = $node->$term_name;
      }
    }
    $rows[] = $row;
  }
  if ($_GET['mode'] == 'csv') {
    die(scouting_csv($header, $rows));
  }
  $output .= '<p>'.l(t('Download results as CSV file'), 'scouting/members?mode=csv').'</p>';
  $output .= theme('table', $header, $rows);

  if ($pager = theme('pager', NULL, $filters['limit'], 0, tablesort_pager())) {
    $output .= $pager;
  }
  return $output;
}

/**
 * Returns a nested (2 level) array of member types (e.g. ASl, CSL, SL, GSL, active member, etc)
 */
function scouting_membership_types($show_all = FALSE) {
  static $types;

  if (!$types) {
    $rs = db_query('SELECT id, name, level FROM {scouting_membership_type}');
    while ($type = db_fetch_object($rs)) {
      $types[$type->level][$type->id] = $type->name;
    }
  }

  if ($show_all) {
    return array(0=>'All') + $types;
  }
  else {
    return $types;
  }
}

?>