<?php
// $Id: section.inc,v 1.4 2006/04/06 03:07:47 unconed Exp $

/**
 * Implementation of hook_access().
 */
function scouting_section_access($op, $node) {
  return _scouting_access($op, $node);
}

/**
 * Implementation of hook_form().
 */
function scouting_section_form(&$node, &$param) {
  scouting_custom_theme();

  $form['title'] = array('#type' => 'textfield', '#title' => t('Section Name'), '#size' => 60, '#maxlength' => 128, '#required' => TRUE, '#default_value' => $node->title, '#description' => t('The name of the section, such as 19th Maidenhead Cubs'));
  $form['section_type_id'] = array('#type' => 'select', '#title' => t('Section Type'), '#default_value' => $node->section_type_id, '#options' => scouting_section_types());
  $form['orgstruct'] = array('#type' => 'textfield', '#title' => t('Organisation Structure'), '#size' => 32, '#maxlength' => 64, '#required' => TRUE, '#default_value' => scouting_orgstruct_name_from_id($node->orgstruct_id), '#autocomplete_path' => 'scouting/orgstruct/autocomplete', '#description' => t('The organisation position of the section (e.g. 19th Maidenhead or ROYAL BERKSHIRE)'));
  $form['leader_name'] = array('#type' => 'textfield', '#title' => t('Leader Name'), '#size' => 65, '#maxlength' => 128, '#required' => TRUE, '#default_value' => scouting_person_name_from_id($node->leader_id), '#autocomplete_path' => 'scouting/people/autocomplete');

  return $form;
}

/**
 * Implementation of hook_load().
 */
function scouting_section_load($node) {
  return db_fetch_object(db_query('SELECT * FROM {scouting_section} WHERE nid = %d', $node->nid));
}

/**
 * Implementation of hook_insert().
 */
function scouting_section_insert($node) {
  db_query('INSERT INTO {scouting_section} (nid, orgstruct_id, section_type_id, leader_id) VALUES (%d, %d, %d, %d)', $node->nid, $node->orgstruct_id, $node->section_type_id, $node->leader_id);
}

/**
 * Implementation of hook_update().
 */
function scouting_section_update($node) {
  db_query('UPDATE {scouting_section} SET orgstruct_id = %d, section_type_id=%d, leader_id = %d WHERE nid = %d', $node->orgstruct_id, $node->section_type_id, $node->leader_id, $node->nid);
}

/**
 * Implementation of hook_delete().
 */
function scouting_section_delete($node) {
  db_query('DELETE FROM {scouting_section} WHERE nid = %d', $node->nid);
}

/**
 * Implementation of hook_view().
 */
function scouting_section_view(&$node, $teaser = false, $page = false) {
  scouting_custom_theme();
}

/**
 * Implementation of hook_validate().
 */
function scouting_section_validate(&$node) {
  if ($node->orgstruct && !$node->orgstruct_id) {
    $node->orgstruct_id = scouting_orgstruct_id_from_name($node->orgstruct);
  }

  if ($node->leader_name && !$node->leader_id) {
    $node->leader_id = scouting_person_id_from_name($node->leader_name);
    if (!$node->leader_id) {
      form_set_error('leader_name', t('Could not find a Scouting person with the given name.'));
    }
  }
}

/**
 * Display a list of all people in the database
 */
function scouting_sections_list_page() {
  scouting_custom_theme();

  $header = array(
    t('Name'),
    t('Section Type'),
    t('Leader'),
    t('Members'),
    t('Action'),
  );

  $sql = 'SELECT n.nid, n.title, p.forename, p.surname, p.prefname, p.nid as "leader_id", st.name AS "section_type", COUNT(*) AS "members" FROM {node} n, {scouting_section} ss, {scouting_section_type} st, {scouting_person} p, {scouting_membership} m WHERE m.section_id = ss.nid AND n.type="scouting_section" AND n.nid=ss.nid AND ss.section_type_id = st.id AND p.nid = ss.leader_id GROUP BY ss.nid ORDER BY n.title';

  $result = db_query($sql);

  while ($node = db_fetch_object($result)) {
    $rows[] = array(
      l($node->title, 'scouting/members', NULL, 'section='.$node->nid),
      $node->section_type,
      l(scouting_person_display_name($node), 'node/'.$node->leader_id),
      $node->members,
      l(t('edit'), 'node/'.$node->nid.'/edit'),
    );
  }

  $output = theme('table', $header, $rows);
  return $output;
}

function scouting_orgstruct_autocomplete($string, $limit) {
  $matches = array();
  $limit = 10;
  $rs = db_query('SELECT name FROM {scouting_orgstruct} WHERE name LIKE "%%%s%%" LIMIT %d', strtolower($string), $limit);
  while ($node = db_fetch_object($rs)) {
    $matches[$node->name] = $node->name;
  }
  echo drupal_to_js($matches);
  exit();
}

function scouting_orgstruct_name_from_id($id) {
  static $cache;
  if (!$cache[$id]) {
    $cache[$id] = db_result(db_query('SELECT name FROM {scouting_orgstruct} WHERE id=%d', $id));
  }
  return $cache[$id];
}

function scouting_orgstruct_id_from_name($name) {
  static $cache;
  if (!$cache[$name]) {
    $cache[$name] = db_result(db_query('SELECT id FROM {scouting_orgstruct} WHERE name="%s"', $name));
  }
  return $cache[$name];
}

function scouting_section_types() {
  static $cache;

  if (!$cache) {
    $rs = db_query('SELECT id, name FROM {scouting_section_type}');
    $cache = array();
    while ($section = db_fetch_object($rs)) {
      $cache[$section->id] = $section->name;
    }
  }

  return $cache;
}

function scouting_sections($show_all = FALSE) {
  static $cache;

  if (!$cache) {
    $rs = db_query('SELECT nid, title FROM {node} WHERE type="scouting_section" ORDER BY title');
    $cache = array();
    while ($node = db_fetch_object($rs)) {
      $cache[$node->nid] = $node->title;
    }
  }

  if ($show_all) {
    $all = array(0=>'All');
    return $cache ? $all + $cache : $all;
  }
  else {
    return $cache;
  }
}

?>